"""Document Approval Models."""
from pydantic import BaseModel, validator, Field
from typing import Optional
from datetime import datetime

import pytz


class Requisition_History(BaseModel):
    id: Optional[str]
    requisitionId: Optional[str]
    action: Optional[str]
    date: Optional[str]
    name: Optional[str] = Field(alias="user")
    passwordAdapter: Optional[str] = Field(alias="user")
    realUser: Optional[str]
    summary: Optional[str]
    uniqueName: Optional[str] = Field(alias="user")

    @validator("name", pre=True, always=True)
    def get_name(cls, value):
        """Get 'name' from 'user' field."""
        if not value:
            return ""
        return value.get("name", "")

    @validator("passwordAdapter", pre=True, always=True)
    def get_password(cls, value):
        """Get 'passwordAdapter' from 'user' field."""
        if not value:
            return ""
        return value.get("passwordAdapter", "")

    @validator("uniqueName", pre=True, always=True)
    def get_uniquename(cls, value):
        """Get 'uniqueName' from 'user' field."""
        if not value:
            return ""
        return value.get("uniqueName", "")

    @validator("date", pre=True, always=True)
    def get_date(cls, value):
        """Transform timestamp date to formatted string date"""
        if value is not None:
            return datetime.fromtimestamp(float(value) / 1e3, tz=pytz.utc).strftime(
                "%Y-%m-%d %H:%M:%S"
            )
        else:
            return ""
