"""Operational Reporting for Sourcing models."""
from datetime import datetime

from pydantic import BaseModel, validator
from typing import Optional


class Invoice_Header(BaseModel):
    Invoice_Header_id: Optional[str]
    CompanyCode: Optional[str]
    CreateDate: Optional[str]
    InvoiceDate: Optional[str]
    InvoiceNumber: Optional[str]
    InvoiceOrigin: Optional[str]
    InvoiceSourceDocument: Optional[str]
    InvoiceSubmissionMethod: Optional[str]
    MaxDiscountApplicable_ApproxAmountInBaseCurrency: Optional[str]
    PaymentTerms: Optional[str]
    ScheduledPaymentDate: Optional[str]
    SubmitDate: Optional[str]
    Supplier_Name: Optional[str]
    TotalInvoicedLessTax_ApproxAmountInBaseCurrency: Optional[str]
    ResubmitDate: Optional[str]
    TimeUpdated: Optional[str]

    @validator(
        "ResubmitDate",
        "ScheduledPaymentDate",
        "SubmitDate",
        "TimeUpdated",
        "CreateDate",
        "InvoiceDate",
        pre=True,
        always=True,
    )
    def get_update_date(cls, value):
        if not value:
            return ""
        return datetime.strptime(value, "%Y-%m-%dT%H:%M:%SZ").strftime(
            "%Y-%m-%d %H:%M:%S"
        )


class Invoice_LineItems(BaseModel):
    Invoice_Header_id: Optional[str]
    Invoice_Item_id: Optional[str]
    Amount_ApproxAmountInBaseCurrency: Optional[str]
    Description_Description: Optional[str]
    Description_SupplierPartNumber: Optional[str]
    Description_UnitOfMeasure: Optional[str]
    Order: Optional[str]
    OrderLineNumber: Optional[str]
    Quantity: Optional[str]
    ShipTo_Name: Optional[str]
    TaxDetail_Description: Optional[str]
    TimeUpdated: Optional[str]

    @validator("TimeUpdated", pre=True, always=True)
    def get_update_date(cls, value):
        if not value:
            return ""
        return datetime.strptime(value, "%Y-%m-%dT%H:%M:%SZ").strftime(
            "%Y-%m-%d %H:%M:%S"
        )


class Invoice_ApprovalRecords(BaseModel):
    Invoice_Header_id: Optional[str]
    Date: Optional[str]
    TimeUpdated: Optional[str]

    @validator("TimeUpdated", "Date", pre=True, always=True)
    def get_update_date(cls, value):
        if not value:
            return ""
        return datetime.strptime(value, "%Y-%m-%dT%H:%M:%SZ").strftime(
            "%Y-%m-%d %H:%M:%S"
        )


class Invoice_ApprovalRequests(BaseModel):
    Invoice_Header_id: Optional[str]
    ActivationDate: Optional[str]
    ManuallyAdded: Optional[str]
    RuleName: Optional[str]
    TimeUpdated: Optional[str]

    @validator("ActivationDate", "TimeUpdated", pre=True, always=True)
    def get_activation_date(cls, value):
        if not value:
            return ""
        return datetime.strptime(value, "%Y-%m-%dT%H:%M:%SZ").strftime(
            "%Y-%m-%d %H:%M:%S"
        )
