"""Operational Reporting for Sourcing models."""
from datetime import datetime

from pydantic import BaseModel, validator
from typing import Optional


class InvoiceReconciliation_Header(BaseModel):
    InvoiceReconciliation_Header_id: Optional[str]
    ApprovedDate: Optional[str]
    CreateDate: Optional[str]
    Invoice: Optional[str]
    PaidDate: Optional[str]
    RejectedDate: Optional[str]
    ResubmitDate: Optional[str]
    StatusString: Optional[str]
    SubmitDate: Optional[str]
    TimeUpdated: Optional[str]

    @validator(
        "TimeUpdated",
        "ApprovedDate",
        "CreateDate",
        "PaidDate",
        "RejectedDate",
        "ResubmitDate",
        "SubmitDate",
        pre=True,
        always=True,
    )
    def get_update_date(cls, value):
        if not value:
            return ""
        return datetime.strptime(value, "%Y-%m-%dT%H:%M:%SZ").strftime(
            "%Y-%m-%d %H:%M:%S"
        )


class InvoiceReconciliation_Payments(BaseModel):
    InvoiceReconciliation_Header_id: Optional[str]
    UniqueName: Optional[str]
    NetDueDate: Optional[str]
    TimeUpdated: Optional[str]

    @validator(
        "NetDueDate",
        "TimeUpdated",
        pre=True,
        always=True,
    )
    def get_update_date(cls, value):
        if not value:
            return ""
        return datetime.strptime(value, "%Y-%m-%dT%H:%M:%SZ").strftime(
            "%Y-%m-%d %H:%M:%S"
        )
