"""Operational Reporting for Procurement_API Models."""
from datetime import datetime

from pydantic import BaseModel, validator
from typing import Optional


class Order_Header(BaseModel):
    Order_Header_id: Optional[str]
    ApprovedDate: Optional[str]
    CompanyCode: Optional[str]
    CreateDate: Optional[str]
    IsServiceOrder: Optional[bool]
    ManuallyOrdered: Optional[bool]
    Name: Optional[str]
    OrderMethodCategory: Optional[str]
    Requester: Optional[str]
    ResubmitDate: Optional[str]
    StatusString: Optional[str]
    Supplier: Optional[str]
    Supplier_Name: Optional[str]
    TimeUpdated: Optional[str]

    @validator(
        "ApprovedDate",
        "CreateDate",
        "ResubmitDate",
        "TimeUpdated",
        pre=True,
        always=True,
    )
    def get_update_date(cls, value):
        if not value:
            return ""
        return datetime.strptime(value, "%Y-%m-%dT%H:%M:%SZ").strftime(
            "%Y-%m-%d %H:%M:%S"
        )


class Order_LineItem(BaseModel):
    Order_Header_id: Optional[str]
    Order_Item_id: Optional[str]
    Amount_ApproxAmountInBaseCurrency: Optional[str]
    BillingAddress_City: Optional[str]
    BillingAddress_Country: Optional[str]
    Description_CommonCommodityCode_Name: Optional[str]
    Description_Description: Optional[str]
    Description_SupplierPartNumber: Optional[str]
    Description_UnitOfMeasure: Optional[str]
    EndDate: Optional[str]
    NeedBy: Optional[str]
    NumberConfirmedAccepted: Optional[str]
    NumberOnReq: Optional[str]
    Quantity: Optional[str]
    Requisition: Optional[str]
    ShipTo_Lines: Optional[str]
    ShipTo_Name: Optional[str]
    TaxDetail_FormulaString: Optional[str]
    TimeUpdated: Optional[str]

    @validator(
        "EndDate",
        "NeedBy",
        "TimeUpdated",
        pre=True,
        always=True,
    )
    def get_update_date(cls, value):
        if not value:
            return ""
        return datetime.strptime(value, "%Y-%m-%dT%H:%M:%SZ").strftime(
            "%Y-%m-%d %H:%M:%S"
        )


class Order_SplitAccountings(BaseModel):
    Order_Header_id: Optional[str]
    Order_Item_id: Optional[str]
    CostCenter_CostCenterDescription: Optional[str]
    TimeUpdated: Optional[str]

    @validator(
        "TimeUpdated",
        pre=True,
        always=True,
    )
    def get_update_date(cls, value):
        if not value:
            return ""
        return datetime.strptime(value, "%Y-%m-%dT%H:%M:%SZ").strftime(
            "%Y-%m-%d %H:%M:%S"
        )
