"""Operational Reporting for Procurement_API Models."""
from datetime import datetime

from pydantic import BaseModel, validator
from typing import Optional


class Receipt_Header(BaseModel):
    Receipt_Header_id: Optional[str]
    ApprovedDate: Optional[str]
    Order: Optional[str]
    Preparer: Optional[str]
    TimeUpdated: Optional[str]

    @validator(
        "ApprovedDate",
        "TimeUpdated",
        pre=True,
        always=True,
    )
    def get_update_date(cls, value):
        if not value:
            return ""
        return datetime.strptime(value, "%Y-%m-%dT%H:%M:%SZ").strftime(
            "%Y-%m-%d %H:%M:%S"
        )


class Receipt_ApprovalRequests(BaseModel):
    Receipt_Header_id: Optional[str]
    ActivationDate: Optional[str]
    ManuallyAdded: Optional[str]
    ReportingReason: Optional[str]
    RuleName: Optional[str]
    LastModified: Optional[str]

    @validator(
        "ActivationDate",
        "LastModified",
        pre=True,
        always=True,
    )
    def get_update_date(cls, value):
        if not value:
            return ""
        return datetime.strptime(value, "%Y-%m-%dT%H:%M:%SZ").strftime(
            "%Y-%m-%d %H:%M:%S"
        )


class Receipt_LineItems(BaseModel):
    Receipt_Header_id: Optional[str]
    Receipt_Line_id: Optional[str]
    Comment: Optional[str]
    Date: Optional[str]
    LineItem_NumberInCollection: Optional[str]
    TimeUpdated: Optional[str]

    @validator(
        "Date",
        "TimeUpdated",
        pre=True,
        always=True,
    )
    def get_update_date(cls, value):
        if not value:
            return ""
        return datetime.strptime(value, "%Y-%m-%dT%H:%M:%SZ").strftime(
            "%Y-%m-%d %H:%M:%S"
        )
