"""Operational Reporting for Procurement_API Models."""
from datetime import datetime
from pydantic import BaseModel, validator
from typing import Optional

from models.document_approval.requisition_history import Requisition_History


class Requisition_Header(BaseModel):
    Requisition_Header_id: Optional[str]
    CreateDate: Optional[str]
    IsServiceRequisition: Optional[str]
    Preparer: Optional[str]
    Requester: Optional[str]
    StatusString: Optional[str]
    TimeUpdated: Optional[str]

    @validator(
        "CreateDate",
        "TimeUpdated",
        pre=True,
        always=True,
    )
    def get_update_date(cls, value):
        if not value:
            return ""
        return datetime.strptime(value, "%Y-%m-%dT%H:%M:%SZ").strftime(
            "%Y-%m-%d %H:%M:%S"
        )


class Requisition_ApprovalRecords(BaseModel):
    Requisition_Header_id: Optional[str]
    Comment_Text: Optional[str]
    Date: Optional[str]
    RealUser: Optional[str]
    User: Optional[str]
    TimeUpdated: Optional[str]

    @validator(
        "Date",
        "TimeUpdated",
        pre=True,
        always=True,
    )
    def get_update_date(cls, value):
        if not value:
            return ""
        return datetime.strptime(value, "%Y-%m-%dT%H:%M:%SZ").strftime(
            "%Y-%m-%d %H:%M:%S"
        )


class Requisition_LineItems(BaseModel):
    Requisition_Header_id: Optional[str]
    Requisition_Item_id: Optional[str]
    Amount_ApproxAmountInBaseCurrency: Optional[str]
    Description_Description: Optional[str]
    Description_CommonCommodityCode_Name: Optional[str]
    Description_ShortName: Optional[str]
    Description_UnitOfMeasure: Optional[str]
    PurchaseGroup: Optional[str]
    PurchaseOrg: Optional[str]
    Quantity: Optional[str]
    TaxDetail_FormulaString: Optional[str]
    TimeUpdated: Optional[str]

    @validator(
        "TimeUpdated",
        pre=True,
        always=True,
    )
    def get_update_date(cls, value):
        if not value:
            return ""
        return datetime.strptime(value, "%Y-%m-%dT%H:%M:%SZ").strftime(
            "%Y-%m-%d %H:%M:%S"
        )
