"""Supplier_Data_API_with_Pagination models."""
from datetime import datetime

from pydantic import BaseModel, Field, validator
from typing import Optional


def date_from_timestamp(timestamp):
    timestamp = int(timestamp)
    # Check if is a number
    if isinstance(timestamp, (int, float)):
        # If is bigger than 1e17 then is in nanoseconds
        if timestamp > 1e17:
            date = datetime.fromtimestamp(timestamp / 1e9)
        # if is bigger than 1e12 then is in miliseconds
        elif timestamp > 1e12:
            date = datetime.fromtimestamp(timestamp / 1000)
        # if none of the previous, then is in seconds
        else:
            date = datetime.fromtimestamp(timestamp)
    else:
        date = None

    return date.strftime("%Y-%m-%d %H:%M:%S")


class VendorList(BaseModel):
    ERP_Vendor_ID: Optional[str] = Field(alias="ERP Vendor ID")
    Address_City: Optional[str] = Field(alias="Address - City")
    Address_Country_Code: Optional[str] = Field(alias="Address - Country Code")
    Address_Postal_Code: Optional[str] = Field(alias="Address - Postal Code")
    TimeUpdated: Optional[str] = Field(alias="Last Update Date")

    @validator("TimeUpdated", pre=True, always=True)
    def get_time_updated(cls, value):
        """Get time created from TimeCreated field and formats to ISO 8601."""
        if not value:
            return ""
        return date_from_timestamp(value)
