import httpx
from httpx import Response
from logger import logger_conf

async def on_response(response: Response):
    """ Each response will be logged. """
    log = logger_conf.get_logger("HTTPX CLIENT")
    if response.is_error:
        # If it is an error, it will be logged as such
        await response.aread()
        error_obj = {}
        try:
            error_obj = response.json()
        except:
            pass
        error_obj = error_obj.get("error", None)
        # Not all errors have an 'error' field with more info
        if isinstance(error_obj, dict):
            log.error(f"An error '{response.status_code}' ocurred. '{error_obj.get('code', 'Unknown')}' : '{error_obj.get('message','No info was sent from the API about the error.')}'")
        else:
            log.error(f"An error '{response.status_code}' has ocurred:")
            log.error(error_obj)
            
        


class Client(httpx.AsyncClient):
    """ Class used to make all http calls. """
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        #self.event_hooks['request'] = [on_request]
        self.event_hooks['response'] = [on_response]


    

