import json
from pprint import pprint
from string import Template
from typing import List
import httpx

from pydantic import parse_obj_as

from templates_methods.sync.base import BaseTemplateMethod
from utils.last_succesful_run import SingletonMeta


class PaginatedByNextTokenMethod(BaseTemplateMethod):
    """
    Template method pattern for paginated by nextToken API.

    This class is used to download data from API
    when the API has a pagination by nextToken.
    """

    __metaclass__ = SingletonMeta


    def __init__(self, api_settings: dict):
        super().__init__(api_settings)
        self._body = api_settings.get("body", {}).get("base",{})
        #self._filter_template = None
        #if self._base_params.get("$filter", None) is not None:
        #    self._filter_template = Template(self._base_params["$filter"])
        #self._filters = api_settings.get("params", {}).get("base", {}).get("filters")
        self._base_params.pop("filters", None)

    def generate_url(self, skip):
        """Generate url to download data from API."""
        base_url = self._base_url.substitute(self._base_path)
        #params = self._base_params.copy()
        self._base_params["$skip"] = skip
        #if self._filter_template is not None and bool(self._filters):
        #    self._base_params["$filter"] = self._filter_template.substitute(self._filters)
        #else:
        #    self._base_params.pop("$filter", None)
        return base_url #+ "?"+"&".join(f"{key}={value}" for key,value in params.items())

    async def download_data(self):
        """
        Download data from API using 'nextToken' to handle pagination.
        """
        print("Starting paginated by nextToken template method...")
        skip = 0
        page = 0
        while True:
            url = self.generate_url(skip)
            response = await self.post_async(url, self._body)
            if isinstance(response, httpx.HTTPStatusError):
                self._logger.error(f"Non retriable error occured while retrieving page {page}.")
                break
            
            nextToken = None
            raw_data = json.loads(response.content)
            if isinstance(raw_data, list):
                parsed_data = parse_obj_as(list[self._model], raw_data)
                self.add_tables_data(parsed_data)

                for item in raw_data:
                    if "nextToken" in item.keys():
                        nextToken = item["nextToken"]
                        skip = nextToken
                        page += 1
            if nextToken is None:
                break
            
