from pprint import pprint
import pytest
from models.operational_reporting_procurement.invoice import Invoice_ApprovalRequests, Invoice_Header
from request_handler.client import Client


from models.sourcing.master_data_retrieval_for_sourcing import CurrencyConversionRate
from tests.unit.test_retry_logic import URL


pytest_plugins = ('pytest_asyncio',)


@pytest.mark.asyncio
async def test_float_decimal_point(httpx_mock):    
    httpx_mock.add_response(
        url=URL,
        status_code=200,
        json={
            "FromCurrency":"IDR",
            "Rate": 1.0E-4,
            "TimeCreated": "2020-10-02 10:56:25.175",
            "ToCurrency": "USD"
        }
    )
    #"IDR"	"2020-10-02 10:56:25.175"	"USD"	"1.0E-4"
    async with Client() as client:
        response = await client.get(url=URL)
        objeto = response.json()
        model = CurrencyConversionRate(**objeto)
        pprint(model)
        assert isinstance(model, CurrencyConversionRate)


@pytest.mark.asyncio
async def test_float_decimal_point_2(httpx_mock):    
    httpx_mock.add_response(
        url=URL,
        status_code=200,
        json={
            "FromCurrency":"IDR",
            "Rate": "33.2526",
            "TimeCreated": "2020-10-02 10:56:25.175",
            "ToCurrency": "USD"
        }
    )
    async with Client() as client:
        response = await client.get(url=URL)
        objeto = response.json()
        model = CurrencyConversionRate(**objeto)
        pprint(model)
        assert isinstance(model, CurrencyConversionRate)


@pytest.mark.asyncio
async def test_date_Invoice_ApprovalRequests(httpx_mock):    
    httpx_mock.add_response(
        url=URL,
        status_code=200,
        json={
            "Invoice_Header_id":"INVA0021-108",
            "ActivationDate": "2023-11-28T17:04:01Z",
            "ManuallyAdded": "False",
            "RuleName": "Approver"
        }
    )
    async with Client() as client:
        response = await client.get(url=URL)
        objeto = response.json()
        model = Invoice_ApprovalRequests(**objeto)
        pprint(model)
        assert isinstance(model, Invoice_ApprovalRequests)


@pytest.mark.asyncio
async def test_date_Invoice_Header(httpx_mock):    
    httpx_mock.add_response(
        url=URL,
        status_code=200,
        json={
            "Invoice_Header_id": "INVA0021-108",
            "CompanyCode": 3000,
            "CreateDate": "2023-11-28T17:03:18Z",
            "InvoiceDate": "2023-11-28T10:30:00Z",
            "InvoiceNumber": "A0021",
            "InvoiceOrigin": "supplier",
            "InvoiceSourceDocument": "PurchaseOrder",
            "InvoiceSubmissionMethod": "PaperInvoice",
            "MaxDiscountApplicable_ApproxAmountInBaseCurrency": 0,
            "PaymentTerms": "0001",
            "ScheduledPaymentDate": "2023-11-28T10:30:00Z",
            "SubmitDate": "2023-11-28T17:04:01Z",
            "Supplier_Name": "Indus Consulting Inc. (AN)",
            "TotalInvoicedLessTax_ApproxAmountInBaseCurrency": 1741,
            "ResubmitDate": "",
            "TimeUpdated": "2023-12-14T18:34:44Z"
        }
    )
    async with Client() as client:
        response = await client.get(url=URL)
        objeto = response.json()
        model = Invoice_Header(**objeto)
        pprint(model)
        assert isinstance(model, Invoice_Header)



