""" File to contain all constants """

# list of all available models

import sys
import anyio
import httpcore
from httpx import Timeout
import httpx
from models.document_approval.requisition_history import Requisition_History
from models.master_data.master_data import CompanyCodes, Countries, Paymentterms, PurchaseGroups, PurchaseOrgs, Suppliers, Uoms, Users
from models.operational_reporting_procurement.invoice import Invoice_ApprovalRecords, Invoice_ApprovalRequests, Invoice_Header, Invoice_LineItems
from models.operational_reporting_procurement.invoice_reconciliation import InvoiceReconciliation_Header, InvoiceReconciliation_Payments
from models.operational_reporting_procurement.order import Order_Header, Order_LineItem, Order_SplitAccountings
from models.operational_reporting_procurement.receipt import Receipt_ApprovalRequests, Receipt_Header, Receipt_LineItems
from models.operational_reporting_procurement.requisition import Requisition_ApprovalRecords, Requisition_Header, Requisition_LineItems
from models.sourcing.master_data_retrieval_for_sourcing import CurrencyConversionRate

from models.sourcing.operational_reporting_for_sourcing import SourcingProject, SourcingProjectDelta
from models.sourcing.sourcing_project_managment import SourcingProjectDetail, SourcingProjectEvent, SourcingProjectsHistory
from models.supplier_data.supplier import VendorList
from output_handler.csv import CSVBase
from tables.document_approval.requisition_history import REQUISITION_HISTORY
from tables.master_data.master_data import COMPANY_CODES, COUNTRIES, PAYMENT_TERMS, PURCHASE_GROUPS, PURCHASE_ORGS, SUPPLIERS, UOMS, USERS
from tables.operational_reporting_procurement.invoice import INVOICE_APPROVAL_RECORDS, INVOICE_APPROVAL_REQUESTS, INVOICE_HEADER, INVOICE_LINE_ITEMS
from tables.operational_reporting_procurement.invoice_reconciliation import INVOICE_RECONCILIATION_HEADER, INVOICE_RECONCILIATION_PAYMENTS
from tables.operational_reporting_procurement.order import ORDER_HEADER, ORDER_LINE_ITEMS, ORDER_SPLIT_ACCOUNTINGS
from tables.operational_reporting_procurement.receipt import RECEIPT_APPROVAL_REQUESTS, RECEIPT_HEADER, RECEIPT_LINE_ITEMS
from tables.operational_reporting_procurement.requisition import REQUISITION_APPROVAL_RECORDS, REQUISITION_HEADER, REQUISITION_LINE_ITEMS
from tables.sourcing.master_data_retrieval_for_sourcing import CURRENCY_CONVERSION_RATE_TABLE
from tables.sourcing.operational_reporting_for_sourcing import SOURCING_PROJECT_TABLE
from tables.sourcing.sourcing_project_managment import SOURCING_PROJECT_DETAILS_TABLE, SOURCING_PROJECT_EVENTS_TABLE, SOURCING_PROJECTS_HISTORY_TABLE
from tables.supplier_data.supplier import VENDOR_LIST
from templates_methods.sync.count_before import CountBeforeTemplateMethod
from templates_methods.sync.counted import CountedTemplateMethod
from templates_methods.sync.list_detail import ListDetailMethod
from templates_methods.sync.listed import ListedMethod
from templates_methods.sync.paginated_by_count import PaginatedByCountMethod
from templates_methods.sync.paginated_by_nextToken import PaginatedByNextTokenMethod
from templates_methods.sync.paginated_by_token import PaginatedByTokenMethod
from templates_methods.async_methods.zipped_file import ZippedFileTemplateMethod
from view_parsers.invoice import Invoice_parser
from view_parsers.invoice_reconciliation import Invoice_reconciliation_parser
from view_parsers.order import Orders_parser
from view_parsers.receipt import Receipt_parser
from view_parsers.requisition import Requisition_parser


BIG_INTEGER = sys.maxsize

TIMEOUT_EXCEPTIONS = (
    httpx.ConnectTimeout,
    httpcore.ConnectTimeout,
    httpx.ConnectError,
    httpcore.ConnectError,
    httpx.TimeoutException,
)
HTTP_EXCEPTIONS = (httpx.HTTPStatusError, httpx.HTTPError)
RETRIABLE_EXPCETIONS = TIMEOUT_EXCEPTIONS + HTTP_EXCEPTIONS

DELTA_MODELS = ["copy_order_updated_range_async",
    "copy_order_updated_range_sync",
    "direct_order_updated_range_async",
    "direct_order_updated_range_sync",
    "erp_order_updated_range_async",
    "erp_order_updated_range_sync",
    "invoice_reconciliation_updated_range_async",
    "invoice_reconciliation_updated_range_sync",
    "invoice_updated_range_async",
    "invoice_updated_range_sync",
    "receipt_updated_range_async",
    "receipt_updated_range_sync",
    "requisition_updated_range_with_history_async",
    "requisition_updated_range_with_history_sync",
    "counted_copy_order_updated_range",
    "counted_direct_order_updated_range",
    "counted_erp_order_updated_range",
    "counted_invoice_reconciliation_updated_range",
    "counted_invoice_updated_range",
    "counted_receipt_updated_range",
    "counted_requisition_updated_range_with_history",
    "sourcing_project_events_sync",
    "sourcing_project_details_sync",
    "sourcing_projects_sync"]

SOURCING_MODELS = ["sourcing_project_events_sync",
                   "sourcing_project_details_sync",
                   "sourcing_projects_history_sync",
                   "sourcing_projects_sync",
                   "currency_conversion_rate"]



P2P_MODELS = ["company_codes",
            "countries",
            "counted_copy_order_updated_range",
            "counted_direct_order_updated_range",
            "counted_erp_order_updated_range",
            "counted_invoice_reconciliation_updated_range",
            "counted_invoice_updated_range",
            "counted_receipt_updated_range",
            "counted_requisition_updated_range_with_history",
            "payment_terms",
            "purchase_groups",
            "purchase_orgs",
            "suppliers",
            "uoms",
            "users",
            "vendor_list"]


TIMEOUT = Timeout(60, connect=60)

MODELS_MAPPING = {
    "Sourcing_projects": SourcingProjectDelta,
    #"Sourcing_projects_delta": SourcingProjectDelta,
    "Sourcing_project_events": SourcingProjectEvent,
    "Sourcing_project_details": SourcingProjectDetail,
    "Sourcing_projects_history": SourcingProjectsHistory,
    "Currency_conversion_rates": CurrencyConversionRate,
    "Order_Header": Order_Header,
    "Order_LineItems": Order_LineItem,
    "Order_SplitAccountings": Order_SplitAccountings,    
    "Requisition_Header": Requisition_Header,
    "Requisition_ApprovalRecords": Requisition_ApprovalRecords,
    "Requisition_LineItems": Requisition_LineItems,
    "Requisition_History": Requisition_History,
    "InvoiceReconciliation_Header" : InvoiceReconciliation_Header,
    "InvoiceReconciliation_Payments": InvoiceReconciliation_Payments,
    "Invoice_Header": Invoice_Header, 
    "Invoice_LineItems": Invoice_LineItems,
    "Invoice_ApprovalRecords": Invoice_ApprovalRecords, 
    "Invoice_ApprovalRequests": Invoice_ApprovalRequests,
    "Receipt_Header": Receipt_Header,
    "Receipt_ApprovalRequests": Receipt_ApprovalRequests,
    "Receipt_LineItems": Receipt_LineItems,
    "companycodes": CompanyCodes,
    "countries": Countries,
    "paymentterms": Paymentterms,
    "purchasegroups": PurchaseGroups,
    "purchseorgs": PurchaseOrgs,
    "uoms": Uoms,
    "users": Users,
    "suppliers": Suppliers, 
    "Vendor_List": VendorList
}

TABLES_MAPPING = {
    "Sourcing_projects": SOURCING_PROJECT_TABLE,
    "Sourcing_project_events": SOURCING_PROJECT_EVENTS_TABLE,
    "Sourcing_project_details": SOURCING_PROJECT_DETAILS_TABLE,
    "Sourcing_projects_history": SOURCING_PROJECTS_HISTORY_TABLE,
    "Currency_conversion_rates": CURRENCY_CONVERSION_RATE_TABLE,
    "Order_Header": ORDER_HEADER,
    "Order_LineItems": ORDER_LINE_ITEMS,
    "Order_SplitAccountings": ORDER_SPLIT_ACCOUNTINGS,
    "Requisition_Header": REQUISITION_HEADER,
    "Requisition_ApprovalRecords": REQUISITION_APPROVAL_RECORDS,
    "Requisition_LineItems": REQUISITION_LINE_ITEMS,
    "Requisition_History": REQUISITION_HISTORY,
    "InvoiceReconciliation_Header" : INVOICE_RECONCILIATION_HEADER,
    "InvoiceReconciliation_Payments": INVOICE_RECONCILIATION_PAYMENTS,
    "Invoice_Header": INVOICE_HEADER, 
    "Invoice_LineItems": INVOICE_LINE_ITEMS, 
    "Invoice_ApprovalRecords": INVOICE_APPROVAL_RECORDS, 
    "Invoice_ApprovalRequests": INVOICE_APPROVAL_REQUESTS,
    "Receipt_Header": RECEIPT_HEADER,
    "Receipt_ApprovalRequests": RECEIPT_APPROVAL_REQUESTS,
    "Receipt_LineItems": RECEIPT_LINE_ITEMS,
    "companycodes": COMPANY_CODES,
    "countries": COUNTRIES,
    "paymentterms": PAYMENT_TERMS,
    "purchasegroups": PURCHASE_GROUPS,
    "purchseorgs": PURCHASE_ORGS,
    "uoms": UOMS,
    "users": USERS,
    "suppliers": SUPPLIERS, 
    "Vendor_List": VENDOR_LIST
}

METHODS_MAPPING = {
    "counted": CountedTemplateMethod,
    "list_detail": ListDetailMethod,    
    "paginated_by_token": PaginatedByTokenMethod,
    "paginated_by_count": PaginatedByCountMethod,
    "paginated_by_next_token": PaginatedByNextTokenMethod,
    "zipped_file_async": ZippedFileTemplateMethod,
    "listed": ListedMethod,
    "count_before": CountBeforeTemplateMethod
}


PARSER_MAPPING = {
    "Requisition_SAP_updatedRange_v2": Requisition_parser,
    "InvoiceReconciliation_SAP_updatedRange": Invoice_reconciliation_parser,
    "Invoice_SAP_updatedRange_v2": Invoice_parser,
    "Receipt_SAP_updatedRange": Receipt_parser,
    "CopyOrder_SAP_updatedRange": Orders_parser,
    "DirectOrder_SAP_updatedRange": Orders_parser,
    "ERPOrder_SAP_updatedRange": Orders_parser
}

OUTPUT_HANDLERS_MAPPING = {
    "csv": CSVBase,
}


FILTERS_MAPPING_USER = {
    "dateFrom": "date-from",
    "dateTo": "date-to"
}


LAST_RUN_FILE = "last_run.json"