""" File that groups functions related to file handling."""

import zipfile, io, json, os
from logger import logger_conf


def full_path_file(file_folder: str, file_name: str):
    """ Joins a folder and a file with the os separator character."""
    return os.path.join(file_folder, file_name)


def file_exists(file_path: str):
    """ Checks if a file/folder exists."""
    return os.path.exists(file_path)


def rename(src_file_name: str, dst_file_name: str):
    """ Renames a file/folder."""
    os.rename(src_file_name, dst_file_name)


def unzip_file(temp_file, folder):
    """ Unzips a file to the desired folder."""
    with open(temp_file.name, mode="rb") as f:
        content = f.read()
        z = zipfile.ZipFile(io.BytesIO(content))
        return z.extractall(folder)


def create_folders(folder_name: str):
    """ Creates a folder."""
    try:
        os.makedirs(folder_name)
    except:
        pass


def get_data_from_file(filepath, mode: str = "r", encoding: str = "utf-8"):
    """ Reads a file with json data and creates an object from it."""
    logger = logger_conf.get_logger("File handler")
    try:
        with open(filepath, mode, encoding=encoding) as f:
            data = json.load(f)
        return data
    except json.JSONDecodeError() as e:
        logger.error(f"Reading line {e.lineno}")
        logger.error(f"{e.msg}")
