from typing import Optional, TypedDict
from datetime import datetime
import json 
import utils.constants as constants

class Filters(TypedDict):
    """ Class to contain all filters and then pass it on to each template method as needed"""
    create_date_from: Optional[str]
    create_date_to: Optional[str]
    update_date_from: Optional[str]
    update_date_to: Optional[str]

class SingletonMeta(type):
    """
    The Singleton class can be implemented in different ways in Python. Some
    possible methods include: base class, decorator, metaclass. We will use the
    metaclass because it is best suited for this purpose.
    """

    _instances = {}

    def __call__(cls, *args, **kwargs):
        """
        Possible changes to the value of the `__init__` argument do not affect
        the returned instance.
        """
        if cls not in cls._instances:
            instance = super().__call__(*args, **kwargs)
            cls._instances[cls] = instance
        return cls._instances[cls]


class LastRunFileManager():
    def save_status(self, model_name, status):
        model_name = self.stardardize_model_name(model_name)

        status_data = self.load_status_file()
        status_data[model_name] = status
        with open(constants.LAST_RUN_FILE,"w") as status_file:
            json.dump(status_data, status_file, indent=2)


    def load_status_file(self):
        try:
            with open(constants.LAST_RUN_FILE, "r") as status_file:
                status_data = json.load(status_file)
                return status_data
        except FileNotFoundError:
            return {}
        
    def get_status(self, model_name):
        model_name = self.stardardize_model_name(model_name)
        status_data = self.load_status_file()
        return status_data.get(model_name, None)
    
    def stardardize_model_name(self, model_name: str):
        """ Standardize model names, because sync and async methods download the same data."""
        model_name = model_name.replace("_async", "")
        model_name = model_name.replace("_sync", "")
        model_name = model_name.replace("counted_", "")
        return model_name
    
        

class LastSuccessfulRun(metaclass = SingletonMeta):
    def __init__(self, model_name):
        self.model_name:str = model_name
        self.latest_update = None

    def set_latest_update(self, date: datetime):
        if self.latest_update is not None:
            if date > self.latest_update:
                self.latest_update = date
        else:
            self.latest_update = date

    def get_run_status(self):
        return dict(latest_update = self.latest_update.strftime("%Y-%m-%dT%H:%M:%SZ"))

