""" File that contains functions to compose two or more yaml files using the !include tag."""
from ruamel import yaml


def compose_documents(self):
    """ Custom function that composes two documents."""
    self.get_event()
    node = self.compose_node(None, None)
    self.get_event()
    return node


def my_safe_load(stream, Loader=yaml.SafeLoader, master=None):
    """ Custom function that loads a yaml file."""
    loader = Loader(stream)
    if master is not None:
        loader.anchors = master.anchors
    try:
        return loader.get_single_data()
    finally:
        loader.dispose()


def yaml_include(loader, node):
    """ Function called whenever we encounter a !include tag."""
    with open(node.value) as inputfile:
        return list(my_safe_load(inputfile, master=loader).values())[0]


def load_composed_yaml(filename):
    """ Function that should be called to read a yaml file that has !include tags in it."""
    yaml.SafeLoader.compose_document = compose_documents
    yaml.add_constructor("!include", yaml_include, Loader=yaml.SafeLoader)

    with open(filename) as fp:
        data = my_safe_load(fp)

    return data
