
from models.operational_reporting_procurement.invoice import Invoice_ApprovalRecords, Invoice_ApprovalRequests, Invoice_Header, Invoice_LineItems
from utils.object_parsing import get_hierarchy
from view_parsers.base import BaseParser


class Invoice_parser(BaseParser):
    """ Parser for a Invoice_SAP_updatedRange_v2 viewTemplate."""
    @classmethod
    def parse_item(cls, item):
        Invoice_Header_id = item.get("UniqueName", None)
        TimeUpdated = item.get("TimeUpdated", None)
        header = {}
        header["Invoice_Header_id"] = Invoice_Header_id
        header["CompanyCode"] = get_hierarchy(item, "CompanyCode", "UniqueName")
        header["CreateDate"] = item.get("CreateDate", None)
        header["InvoiceDate"] = item.get("InvoiceDate", None)
        header["InvoiceNumber"] = item.get("InvoiceNumber", None)
        header["InvoiceOrigin"] = item.get("InvoiceOrigin", None)
        header["InvoiceSourceDocument"] = item.get("InvoiceSourceDocument", None)
        header["InvoiceSubmissionMethod"] = item.get("InvoiceSubmissionMethod", None)
        header["MaxDiscountApplicable_ApproxAmountInBaseCurrency"] = get_hierarchy(item, "MaxDiscountApplicable", "ApproxAmountInBaseCurrency")
        header["PaymentTerms"] = get_hierarchy(item, "PaymentTerms", "UniqueName")
        header["ScheduledPaymentDate"] = item.get("ScheduledPaymentDate", None)
        header["SubmitDate"] = item.get("SubmitDate", None)
        header["Supplier_Name"] = get_hierarchy(item, "Supplier", "Name")
        header["TotalInvoicedLessTax_ApproxAmountInBaseCurrency"] = get_hierarchy(item, "TotalInvoicedLessTax", "ApproxAmountInBaseCurrency")
        header["ResubmitDate"] = item.get("ResubmitDate", None)
        header["TimeUpdated"] = TimeUpdated

        Header = Invoice_Header(**header)


        line_items = []
        LineItems = item.get("LineItems", [])
        if LineItems is not None:
            for line_item in LineItems:
                temp_item = {}
                temp_item["Invoice_Header_id"] = Invoice_Header_id
                temp_item["Invoice_Item_id"] = line_item.get("NumberInCollection", None)
                temp_item["Amount_ApproxAmountInBaseCurrency"] = get_hierarchy(line_item, "Amount", "ApproxAmountInBaseCurrency")
                temp_item["Description_Description"] = line_item.get("Description.Description", None)
                temp_item["Description_SupplierPartNumber"] = line_item.get("Description.SupplierPartNumber", None)
                temp_item["Description_UnitOfMeasure"] = get_hierarchy(line_item, "Description.UnitOfMeasure", "UniqueName")
                temp_item["Order"] = get_hierarchy(line_item,"Order", "UniqueName")

                temp_item["OrderLineNumber"] = line_item.get("OrderLineNumber", None)
                temp_item["Quantity"] = line_item.get("Quantity", None)
                temp_item["ShipTo_Name"] = get_hierarchy(line_item, "ShipTo", "Name")
                temp_item["TaxDetail_Description"] = line_item.get("TaxDetail.Description", None)
                # Each line item doesn't have an updated date, so using main one
                temp_item["TimeUpdated"] = TimeUpdated


                line_items.append(Invoice_LineItems(**temp_item))




        records_list = []   
        ApprovalRecords = item.get("ApprovalRecords", [])
        if ApprovalRecords is not None:
            for record in ApprovalRecords:
                temp_record = {}
                temp_record["Invoice_Header_id"] = Invoice_Header_id
                temp_record["Date"] = record.get("Date", None)
                # each approval record doesn't have an updated date, so using main one
                temp_record["TimeUpdated"] = TimeUpdated

                records_list.append(Invoice_ApprovalRecords(**temp_record))

        requests_list = []
        ApprovalRequests = item.get("ApprovalRequests",[])
        if ApprovalRequests is not None:
            for request in ApprovalRequests:
                temp_approval_req = {}
                temp_approval_req["Invoice_Header_id"] = Invoice_Header_id
                temp_approval_req["ActivationDate"] = request.get("ActivationDate", None)
                temp_approval_req["ManuallyAdded"] = request.get("ManuallyAdded", None)
                temp_approval_req["RuleName"] = request.get("RuleName", None)
                # each approval request doesn't have an updated date, so using main one
                temp_approval_req["TimeUpdated"] = TimeUpdated
                requests_list.append(Invoice_ApprovalRequests(**temp_approval_req))
        

        return {"Invoice_Header": [Header],
                "Invoice_LineItems": line_items,
                "Invoice_ApprovalRecords": records_list ,
                "Invoice_ApprovalRequests": requests_list }