

from models.operational_reporting_procurement.order import Order_Header, Order_LineItem, Order_SplitAccountings
from utils.object_parsing import get_hierarchy
from view_parsers.base import BaseParser


class Orders_parser(BaseParser):
    """ Parser for:
        - CopyOrder_SAP_updatedRange
        - DirectOrder_SAP_updatedRange
        - ERPOrder_SAP_updatedRange 
    viewTemplates."""
    @classmethod
    def parse_item(cls, item):
        Order_Header_id = item.get("UniqueName", None)
        TimeUpdated = item.get("TimeUpdated", None)
        header = {}
        header["Order_Header_id"] = Order_Header_id
        header["ApprovedDate"] = item.get("ApprovedDate", None)
        header["CompanyCode"] = get_hierarchy(item, "CompanyCode", "UniqueName")
        header["CreateDate"] = item.get("CreateDate", None)
        header["IsServiceOrder"] = item.get("IsServiceOrder",None)
        header["ManuallyOrdered"] = item.get("ManuallyOrdered", None)
        header["Name"] = item.get("Name", None)
        header["OrderMethodCategory"] = item.get("OrderMethodCategory", None)
        header["Requester"] = get_hierarchy(item, "Requester", "UniqueName")
        header["ResubmitDate"] = item.get("ResubmitDate", None)
        header["StatusString"] = item.get("StatusString", None)
        header["Supplier"] = get_hierarchy(item, "Supplier", "UniqueName")
        header["Supplier_Name"] = get_hierarchy(item, "Supplier", "Name")
        header["TimeUpdated"] = TimeUpdated

        Header = Order_Header(**header)


        line_items = []
        split_acountings = []
        LineItems = item.get("LineItems", None)
        if LineItems is not None:
            for line_item in LineItems:
                temp_item = {}
                temp_item["Order_Header_id"] = Order_Header_id
                temp_item["Order_Item_id"] = line_item.get("NumberInCollection", None)
                temp_item["Amount_ApproxAmountInBaseCurrency"] = get_hierarchy(line_item, "Amount", "ApproxAmountInBaseCurrency")
                temp_item["BillingAddress_City"] = get_hierarchy(line_item, "BillingAddress", "City")
                country = get_hierarchy(line_item, "BillingAddress", "Country") 
                temp_item["BillingAddress_Country"] = country.get("UniqueName", None) if country is not None else None
                temp_item["Description_CommonCommodityCode_Name"] = get_hierarchy(line_item, "Description.CommonCommodityCode", "Name")
                temp_item["Description_Description"] = line_item.get("Description.Description", None)
                temp_item["Description_SupplierPartNumber"] = line_item.get("Description.SupplierPartNumber", None)
                temp_item["Description_UnitOfMeasure"] = get_hierarchy(line_item, "Description.UnitOfMeasure", "UniqueName")
                temp_item["EndDate"] = line_item.get("EndDate", None)
                temp_item["NeedBy"] = line_item.get("NeedBy", None)
                temp_item["NumberConfirmedAccepted"] = line_item.get("NumberConfirmedAccepted", None)
                temp_item["NumberOnReq"] = line_item.get("NumberOnReq", None)
                temp_item["Quantity"] = line_item.get("Quantity", None)
                temp_item["Requisition"] = get_hierarchy(line_item, "Requisition", "UniqueName")
                temp_item["ShipTo_Lines"] = get_hierarchy(line_item, "ShipTo", "Lines")
                temp_item["ShipTo_Name"] = get_hierarchy(line_item, "ShipTo", "Name")
                temp_item["TaxDetail_FormulaString"] = line_item.get("TaxDetail.FormulaString", None)
                # each line item doesn't have an updated date, so using main one
                temp_item["TimeUpdated"] = TimeUpdated
                
                line_items.append(Order_LineItem(**temp_item))

                SplitAccountings = line_item.get("SplitAccountings", None)
                if SplitAccountings is not None:
                    for accounting in SplitAccountings:
                        temp_accounting = {}
                        temp_accounting["Order_Header_id"] = Order_Header_id
                        temp_accounting["Order_Item_id"] = line_item.get("NumberInCollection", None)
                        temp_accounting["CostCenter_CostCenterDescription"] = get_hierarchy(accounting, "CostCenter", "CostCenterDescription")
                        # each element doesn't have an updated date, so using main one
                        temp_accounting["TimeUpdated"] = TimeUpdated

                        split_acountings.append(Order_SplitAccountings(**temp_accounting))


            
        return {"Order_Header": [Header],
                "Order_LineItems": line_items,
                "Order_SplitAccountings": split_acountings}




