from models.operational_reporting_procurement.receipt import Receipt_ApprovalRequests, Receipt_Header, Receipt_LineItems
from utils.object_parsing import get_hierarchy
from view_parsers.base import BaseParser


class Receipt_parser(BaseParser):
    """ Parser for a Receipt_SAP_updatedRange viewTemplate."""
    @classmethod
    def parse_item(cls, item):
        Receipt_Header_id = item.get("UniqueName", None)
        # Main updated date
        TimeUpdated = item.get("TimeUpdated", None)
        header = {}
        header["Receipt_Header_id"] = Receipt_Header_id
        header["ApprovedDate"] = item.get("ApprovedDate", None)
        header["Order"] = get_hierarchy(item, "Order", "UniqueName")
        header["Preparer"] = get_hierarchy(item, "Preparer", "UniqueName")
        header["TimeUpdated"] = TimeUpdated

        Header = Receipt_Header(**header)


        approvalRequests_list = []
        ApprovalRequests = item.get("ApprovalRequests", None)
        if ApprovalRequests is not None:
            for request in ApprovalRequests:
                temp_request = {}
                temp_request["Receipt_Header_id"] = Receipt_Header_id

                temp_request["ActivationDate"] = request.get("ActivationDate", None)
                temp_request["ManuallyAdded"] = request.get("ManuallyAdded", None)
                temp_request["ReportingReason"] = request.get("ReportingReason", None)
                temp_request["RuleName"] = request.get("RuleName", None)
                # each approval requests has their own update date, so using that
                #although quite similar with the main one
                temp_request["LastModified"] = request.get("LastModified", None)

                approvalRequests_list.append(Receipt_ApprovalRequests(**temp_request))


        line_items = []
        LineItems = item.get("ReceiptItems", None)
        if LineItems is not None:
            for line_item in LineItems:
                temp_item = {}
                temp_item["Receipt_Header_id"] = Receipt_Header_id
                temp_item["Receipt_Line_id"] = line_item.get("NumberInCollection", None)
                temp_item["Comment"] = line_item.get("Comment", None)
                temp_item["Date"] = line_item.get("Date", None)
                temp_item["LineItem_NumberInCollection"] = line_item.get("LineItem.NumberInCollection", None)
                # Each receipt item doesn't have a update date, so using the main one
                temp_item["TimeUpdated"] = TimeUpdated
                line_items.append(Receipt_LineItems(**temp_item))

        return {"Receipt_Header": [Header],
                "Receipt_ApprovalRequests": approvalRequests_list,
                "Receipt_LineItems":line_items}