from models.operational_reporting_procurement.requisition import Requisition_ApprovalRecords, Requisition_Header, Requisition_LineItems
from utils.object_parsing import get_hierarchy
from view_parsers.base import BaseParser


class Requisition_parser(BaseParser):
    """ Parser for a Requisition_SAP_updatedRange_v2 viewTemplate."""
    @classmethod
    def get_id_field(cls):
        return "UniqueName"

    @classmethod
    def parse_item(cls, item):
        header = {}
        id = item.get("UniqueName", None)
        TimeUpdated = item.get("TimeUpdated", None)
        header["Requisition_Header_id"] = id
        header["CreateDate"] = item.get("CreateDate", None)
        header["IsServiceRequisition"] = item.get("IsServiceRequisition", None)
        header["Preparer"] = get_hierarchy(item, "Preparer", "UniqueName") #item.get("Preparer").get("UniqueName", None) if item.get("Preparer", None) is not None else None
        header["Requester"] = get_hierarchy(item, "Requester", "UniqueName") #item.get("Requester").get("UniqueName", None) if item.get("Requester", None) is not None else None
        header["StatusString"] = item.get("StatusString",None)
        header["TimeUpdated"] = TimeUpdated
        Header = Requisition_Header(**header)


        approval_records = item.get("ApprovalRecords",[])
        approval_list = []
        if approval_records:
            for record in approval_records:
                approval = {}
                approval["Requisition_Header_id"] = id            
                approval["Comment_Text"] = get_hierarchy(item, "Comment", "Text") #record.get("Comment").get("Text", None) if record.get("Comment",None) is not None else None
                approval["Date"] = record.get("Date")
                approval["RealUser"] = record.get("RealUser")
                approval["User"] = get_hierarchy(item, "User", "UniqueName") #record.get("User").get("UniqueName", None) if record.get("User", None) is not None else None
                # each Approval Records doesn't have any updated date, so using the main one
                approval["TimeUpdated"] = TimeUpdated
                approval_list.append(Requisition_ApprovalRecords(**approval))

        LineItems = []

        line_items = item.get("LineItems",[])
        if line_items is not None:
            for line_item in line_items:
                temp_item = {}

                temp_item["Requisition_Header_id"] = id
                temp_item["Requisition_Item_id"] = line_item.get("NumberInCollection", None)
                temp_item["Amount_ApproxAmountInBaseCurrency"] = get_hierarchy(item, "Amount", "ApproxAmountInBaseCurrency") #line_item.get("Amount").get("ApproxAmountInBaseCurrency", None)  if line_item.get("Amount",None) is not None else None
                temp_item["Description_Description"] = line_item.get("Description.Description", None)
                temp_item["Description_CommonCommodityCode_Name"] = get_hierarchy(item, "Description.CommonCommodityCode", "Name") #line_item.get("Description.CommonCommodityCode").get("Name", None) if line_item.get("Description.CommonCommodityCode",None) is not None else None
                temp_item["Description_ShortName"] = line_item.get("Description.ShortName", None)
                temp_item["Description_UnitOfMeasure"] = get_hierarchy(item, "Description.UnitOfMeasure", "UniqueName") #line_item.get("Description.UnitOfMeasure").get("UniqueName", None) if line_item.get("Description.UnitOfMeasure", None) is not None else None
                temp_item["PurchaseGroup"] = get_hierarchy(item, "PurchaseGroup", "UniqueName") #line_item.get("PurchaseGroup").get("UniqueName", None) if line_item.get("PurchaseGroup", None) is not None else None
                temp_item["PurchaseOrg"] = get_hierarchy(item, "PurchaseOrg", "UniqueName") #line_item.get("PurchaseOrg").get("UniqueName", None) if line_item.get("PurchaseOrg", None) is not None else None
                temp_item["Quantity"] = line_item.get("Quantity", None)
                temp_item["TaxDetail_FormulaString"] = line_item.get("TaxDetail.FormulaString", None)
                # each lineitem doesn't have a updated date, so using main one
                temp_item["TimeUpdated"] = TimeUpdated

                LineItems.append(Requisition_LineItems(**temp_item))

        #Its important to use as object key the same key of the model it represents
        requisition = {"Requisition_Header": [Header], "Requisition_ApprovalRecords": approval_list,"Requisition_LineItems": LineItems}

        return requisition